########################################################################################################################.
# SACS - small area cancer simulation.
# Last change: 18.06.2004.
# Version: 0.1
# Authors: Martin Radespiel-Trger, M.D., Anja Daugs,
#          Registration office , Population-based cancer registry Bavaria
#          stliche Stadtmauerstr. 29, D-91054 Erlangen, Germany
#          Email: martin.radespiel-troeger@ekr.med.uni-erlangen.de
########################################################################################################################.
#  Copyright (C) 2004  M. Radespiel-Trger, A. Daugs.
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
########################################################################################################################.

if (version$os == "mingw32") {
	memory.limit(size = 1024)
}

library(survival)
#options(warn=2)
options(warn=-1)

############################################################
# draw polygons.
# Author: Denis White <white.denis@epa.gov>
# URL: http://www.epa.gov/wed/pages/staff/white/
############################################################
ngon <- function (xydc, n=4, type=1)
  # draw or fill regular polygon
  # xydc a four element vector with
  #   x and y of center, d diameter in mm, and c color
  # n number of sides of polygon, n>8 => circle
  #   if n odd, vertex at (0,y), else midpoint of side
  # type=1 => interior filled, type=2 => edge
  # type=3 => both
{
  u <- par("usr")
  p <- par("pin")
  d <- as.numeric(xydc[3])
  inch <- d/25.4
  rad <- inch*((u[2]-u[1])/p[1])/2
  ys <- inch*((u[4]-u[3])/p[2])/2/rad
  if (n > 8) n <- d*4 + 1
  th <- pi*2/n
  costh <- cos (th)
  sinth <- sin (th)
  x <- y <- rep (0,n+1)
  if (n %% 2) {
    x[1] <- 0
    y[1] <- rad
    }
  else {
    x[1] <- -rad*sin(th/2)
    y[1] <-  rad*cos(th/2)
    }
  for (i in 2:(n+1)) {
    xl <- x[i-1]
    yl <- y[i-1]
    x[i] <- xl*costh - yl*sinth
    y[i] <- xl*sinth + yl*costh
    }
  x <- x + as.numeric(xydc[1])
  y <- y*ys + as.numeric(xydc[2])
  if (type %% 2) polygon (x,y,col=xydc[4],border=0)
  if (type %/% 2) lines (x,y,col=xydc[4])
  invisible()
}


########################################################################################################################.
# NAME: fsurvprob
# CREATED: 16.02.01
# LAST CHANGE: 16.02.01
# PURPOSE: calculate arbitrary Kaplan Meier survival probability.
# ARGUMENT(S): zeit, zens, tpoint ((Vector of) timepoint(s)), compl (calculate Complement?)
# RETURN: K-M survival probability
########################################################################################################################.
fsurvprob <- function(zeit, zens, tpoint, compl=0) {
	sdat <- data.frame(zeit, zens)
	ypoint <- NA
	kmfit <- survfit(Surv(zeit, zens), type = "kaplan-meier", data = sdat)
	# insert "estimate" for time = 0.
	xvec <- c(0, kmfit$time)
	yvec <- c(1, kmfit$surv)
	# calc number of timepoints for which estimates are required.
	numtpoint <- length(tpoint) 
	if (numtpoint == 1) {
		# estimate surv. prob. either at exact required time point or at last event time before exact time point.
		if (tpoint <= max(xvec)) {
			xpoint <- xvec[xvec <= tpoint]
			if (length(xpoint) > 1) xpoint <- max(xpoint)
			# in case of tied times, select first element.
			if (length(xpoint) > 1) xpoint <- xpoint[1]
			ypoint <- yvec[xvec == xpoint]
			# theoretically unnecessary: in case of tied surv. probs., select first element
			if (length(ypoint) > 1) ypoint <- ypoint[1]
		}
		else ypoint <- NA
	}
	else {
		# define vector that will take the required surv. prob. estimates.
		ypoint <- single(numtpoint)
		for (i in 1:numtpoint) {
			element <- tpoint[i]
			if (element <= max(xvec)) {
				# estimate surv. prob. either at exact required time point or at last event time before exact time point.
				xpoint <- xvec[xvec <= element]
				if (length(xpoint) > 1) xpoint <- max(xpoint)
				# in case of tied survival times, select first element of vector.
				if (length(xpoint) > 1) xpoint <- xpoint[1]
				yp <- yvec[xvec == xpoint]
				# theoretically unnecessary: in case of tied surv. probs., select first element
				if (length(yp) > 1) yp <- yp[1]
				ypoint[i] <- yp
			}
			else ypoint[i] <- NA
		}
	}
	if (compl == 1 && (!is.na(ypoint))) ypoint <- 1 - ypoint
	ypoint
}

########################################################################################################################.
# NAME: fhouse
# CREATED: 
# LAST CHANGE: 
# PURPOSE: 
# ARGUMENT(S): 
# RETURN: 
########################################################################################################################.
fhouse <- function(x, y, farbe, year, age, prevage=age, status, nyears, fillcolvec, textcolvec) {
	# define house
	house <- list(list(c(x+7,    x+9),  c(y,   y+1)), # right lower horizontal house corner
			list(c(x+9,    x+9),  c(y+1, y+8)), # right vertical house corner
			list(c(x+9,    x+7),  c(y+8, y+7)), # right upper horizontal house corner
			list(c(x,    x+3.5),  c(y+7, y+9)), # left front gable margin
			list(c(x+3.5,  x+7),  c(y+9, y+7)), # right front gable margin
			list(c(x+3.5,x+5.5),  c(y+9,y+10)), # roof ridge
			list(c(x+5.5,  x+9),  c(y+10,y+8))) # right rear gable margin
	# assign 'nice' drawing parameters.
	leftmargin <- 0.5
	lowermargin <- 0.3
	xboxdist <- 2.2
	yboxdist <- 1.7
	xboxsize <- 1.8
	yboxsize <- 1.1
	xtextoffset <- 0.85
	ytextoffset <- 0.55
	nboxperrow <- 3
	# assign start coordinates for 'deleting' previous persons.
	currx <- x + leftmargin
	curry <- y + lowermargin
	xcount <- 0
	# delete previous graph if year > 1 and if number of house inhabitants has decreased since last year.
	if (year > 1 && length(age) < length(prevage)) {
		i <- 1
		while (i <= length(prevage)) {
			# overwrite previous rectangles with white-filled rectangles.
			rect(currx, curry, currx+xboxsize, curry+yboxsize, border="white", col="white")
			xcount <- xcount + 1
			currx <- currx + xboxdist
			if (xcount == nboxperrow) {
				curry <- curry + yboxdist
				currx <- x + leftmargin
				xcount <- 0
			}
			i <- i + 1
		}
	}
	if (year == 1 || year == nyears || (length(age) < length(prevage))) {
		# draw house.
		rect(x,y,x+7,y+7, border="black")
		for (i in 1:7) lines(house[[i]][[1]], house[[i]][[2]], col="black")
	}
	# assign start coordinates for drawing persons
	currx <- x + leftmargin
	curry <- y + lowermargin
	xcount <- 0
	# plot new graph (fill house with rectangles and printed age).
	#ngon <- function (xydc, n=4, type=1)
	# draw or fill regular polygon
	# xydc a four element vector with
	#   x and y of center, d diameter in mm, and c color
	# n number of sides of polygon, n>8 => circle
	#   if n odd, vertex at (0,y), else midpoint of side
	# type=1 => interior filled, type=2 => edge
	# type=3 => both
	i <- 1
	while (i <= length(age)) {
		# print symbols.
		ngon.type <- 3 # filled ngon
		ngon.col <- fillcolvec[status[i]]
		if (fillcolvec[status[i]] == "white") {
			ngon.type <- 2 # ngon edge only
			ngon.col <- "black"
		}
		ngon.nedge <- 4
		ngon.d <- 4.3
		if (status[i] > 1 && status[i] < 6) {
			ngon.nedge <- 10
			ngon.d <- 3.5
		}
		ngon(xydc=c(currx+0.8, curry+0.6, ngon.d, ngon.col), n=ngon.nedge, type=ngon.type)
		#testage <- NULL
		#try(testage <- age[i])
		#if (is.null(testage)) cat("status[i]:",  status[i], ", age[i]:", age[i], "\n")
		#options(show.error.messages=F)
		# print person age.
		try(text(currx+xtextoffset, curry+ytextoffset, labels=age[i], cex=0.6, col=textcolvec[status[i]]))
		#options(show.error.messages=T)
		xcount <- xcount + 1
		currx <- currx + xboxdist
		if (xcount == nboxperrow) {
			curry <- curry + yboxdist
			currx <- x + leftmargin
			xcount <- 0
		}
		i <- i + 1
	}
}

#######################################################################################################################.
# NAME: sacs
# CREATED: 
# LAST CHANGE: 
# PURPOSE: 
# ARGUMENT(S): 
# RETURN: 
########################################################################################################################.
sacs <- function(startn=500, nhouse=100, nyears=1, incid=0.0035, cancrelmort=0.0025, noncancrelmort=0.0075, 
				birthproc=T, brate=0.01, memtime=5, villagegraph=T, numericgraph=T, runintime=200, graphfiles=F,
				graphtype=NULL) {

	verbose <- 0
	print.weights <- F

	if (startn > 1000 && villagegraph) stop("maximum start number of inhabitants is 1000 for village graphic mode.")
	if (nhouse > 100) stop("maximum number of houses is 100.")
	if (villagegraph && numericgraph) stop("only one graph type allowed in current version.")

	###########################################################################################################################################################
	# assign weight table for age-weighted sampling of cancer incidence.
	# AD: gewichtetes Sampling der Neuerkrankungen erfolgt ber altersspezifische Inzidenzraten des Saarlands
	###########################################################################################################################################################
	agerelincvec <- c(mean(28.63,15.63), mean(4.67,9.87), mean(10.20,7.17), mean(18.47,17.20), mean(32.87,19.87), mean(38.33,38.43), mean(55.53,82.83), 
				mean(98.43,148.10), mean(154.67,261.23), mean(293.57,333.50),	mean(467.87,519.10), mean(811.90,696.23), mean(1175.43,769.83), 
				mean(1643.37,878.50), mean(2042.17,1133.07), mean(2187.87,1300.70), mean(2333.40,1510.77), mean(2251.97,1487.83)) 
	incidweights <- agerelincvec / sum(agerelincvec)
	incidweighttab <- data.frame(min=c(0,5,10,15,20,25,30,35,40,45,50,55,60,65,70,75,80,85), 
				  	max=c(4,9,14,19,24,29,34,39,44,49,54,59,64,69,74,79,84,1000), weights=incidweights)
	#print("incidweighttab$weights")
	#cat(round(incidweighttab$weights, digits=5), "\n")
	if (print.weights) {
		print("incidweighttab")
		print(incidweighttab)
	}
	###########################################################################################################################################################
	# assign weight table for age-weighted sampling of cancer-related death.
	# AD: gewichtetes Sampling der Krebstodesflle erfolgt ber die 1-Jahr-Sterbenswahrscheinlichkeit (ermittelt aus 1-Jahresberlebensrate des Saarlands)
	###########################################################################################################################################################
	agerelcrmortvec <- c(5,14,26,33,37,48,85) 
	crmortweights <- agerelcrmortvec / sum(agerelcrmortvec)
	cancreldeathweighttab <- data.frame(min=c(0,15,45,55,65,75,100), 
						  max=c(14,44,54,64,74,99,1000), weights=crmortweights)
	#print("cancreldeathweighttab$weights")
	#cat(round(cancreldeathweighttab$weights, digits=5), "\n")
	if (print.weights) {
		print("cancreldeathweighttab")
		print(cancreldeathweighttab)
	}
	###########################################################################################################################################################
	# assign weight table for age-weighted sampling of non-cancer-related death. 
	# AD: gewichtetes Sampling der Nichtkrebstodesflle erfolgt ber die bayerische Sterbenswahrscheinlichkeit (entnommen aus der bayerischen Sterbetafel)	
	###########################################################################################################################################################
	agerelncrmortvec <- c(sum(0.004365,0.00034,0.00024,0.000175,0.00014),  #0-4
				sum(0.000125,0.000125,0.000115,0.000105,0.0001),  #5-9
				sum(0.0001,0.000115,0.00014,0.000165,0.00021),    #10-14
				sum(0.000285,0.0004,0.00054,0.00067,0.00075),     #15-19
				sum(0.00077,0.00074,0.00069,0.00064,0.00061),     #20-24
				sum(0.0006,0.000605,0.000615,0.000615,0.00062),   #25-29
				sum(0.000635,0.000665,0.000695,0.00075,0.00083),  #30-34
				sum(0.00092,0.001005,0.001105,0.00121,0.00134),   #35-39
				sum(0.00149,0.00166,0.001835,0.00202,0.002215),   #40-44
				sum(0.002425,0.00265,0.002885,0.00316,0.00346),   #45-49
				sum(0.0038,0.004165,0.004555,0.00496,0.00539),    #50-54
				sum(0.005855,0.006365,0.00693,0.007565,0.008285), #55-59
				sum(0.009095,0.01001,0.011025,0.01215,0.013385),  #60-64
				sum(0.01474,0.01622,0.01785,0.01965,0.021635),    #65-69
				sum(0.023865,0.026365,0.02919,0.03238,0.03598),   #70-74
				sum(0.040045,0.044625,0.049785,0.055575,0.062055),#75-79
				sum(0.06829,0.07732,0.086205,0.09598,0.1067),     #80-84
				sum(0.118405,0.13112,0.144885,0.159715,0.17562),  #85-89
				sum(0.19261,0.21067,0.22978,0.2499,0.271135),     #90-94
				sum(0.29334,0.3164,0.340635,0.36591,0.39222),     #95-99
				sum(0.419565,0.9,0.9,0.9,0.9))                    #100-1000  
	ncrmortweights <- agerelncrmortvec / sum(agerelncrmortvec)
	noncancreldeathweighttab <- data.frame(min = c(0,5,10,15,20,25,30,35,40,45,50,55,60,65,70,75,80,85,90,95,100), 
				  			max=c(4,9,14,19,24,29,34,39,44,49,54,59,64,69,74,79,84,89,94,99,1000), weights=ncrmortweights)
	# old version; too much detail for our purpose.
	#agerelncrmortvec <- c(0.004365,0.00034,0.00024,0.000175,0.00014,0.000125,0.000125,0.000115,0.000105,0.0001,0.0001,0.000115,0.00014,0.000165,0.00021,
	#				0.000285,0.0004,0.00054,0.00067,0.00075,0.00077,0.00074,0.00069,0.00064,0.00061,0.0006,0.000605,0.000615,0.000615,0.00062,
	#				0.000635,0.000665,0.000695,0.00075,0.00083,0.00092,0.001005,0.001105,0.00121,0.00134,0.00149,0.00166,0.001835,0.00202,0.002215,
	#				0.002425,0.00265,0.002885,0.00316,0.00346,0.0038,0.004165,0.004555,0.00496,0.00539,0.005855,0.006365,0.00693,0.007565,0.008285,
	#				0.009095,0.01001,0.011025,0.01215,0.013385,0.01474,0.01622,0.01785,0.01965,0.021635,0.023865,0.026365,0.02919,0.03238,0.03598,
	#				0.040045,0.044625,0.049785,0.055575,0.062055,0.06829,0.07732,0.086205,0.09598,0.1067,0.118405,0.13112,0.144885,0.159715,0.17562,
	#				0.19261,0.21067,0.22978,0.2499,0.271135,0.29334,0.3164,0.340635,0.36591,0.39222,0.419565,0.5)
	#ncrmortweights <- agerelncrmortvec / sum(agerelncrmortvec)
	#noncancreldeathweighttab <- data.frame(min=c(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,
	#							41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,
	#							81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96,97,98,99,100,101), 
	#					  max=c(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,
	#							41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,
	#							81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96,97,98,99,100,1000), weights=ncrmortweights)
	#print("noncancreldeathweighttab$weights")
	#cat(round(noncancreldeathweighttab$weights, digits=5), "\n")
	if (print.weights) {
		print("noncancreldeathweighttab")
		print(noncancreldeathweighttab)
	}
	###########################################################################################################################################################
      # assign weight table for age distribution of start population 
	# AD: das Alter der Startpopulation ist nicht mehr standardnormalverteilt, sondern entspricht der bayerischen Bevlkerungsstruktur
	###########################################################################################################################################################
      agerelpopdisvec <- c(117255, 121250, 125243, 128323, 130243, 130057, 129528, 131587, 135172, 138648, 141604, 142297, 142839, 139412, 137582, 132972, 130937, 129926, 
					132901, 137124, 141034, 141263, 140320, 139516, 140781, 141146, 142680, 144490, 151410, 161966, 175064, 188644, 200379, 210497, 216655, 
					219286, 221302, 221349, 219532, 215569, 210013, 204336, 196663, 189902, 183327, 176637, 170661, 164850, 162792, 160044, 160251, 
					158876, 156878, 153227, 145086, 130695, 127687, 130430, 140194, 149325, 162134, 174858, 172392, 161581, 150105, 141368, 
					133590, 121182, 111385, 104926, 106041, 105651, 104992, 100620, 97034, 92915, 88571, 84211, 80778, 79506, 76670, 
					67008, 50831, 36409, 28580, 40021, 36686, 31362, 26273, 21659, 17488, 13802, 10632, 7975, 5813, 4107, 2807, 1851, 1174, 713, 414)
      popclasssizeweights <- agerelpopdisvec / sum(agerelpopdisvec)
	########################################
	# assign status codes.
	########################################
	healthcode <- 1			
	incidcode <- 2			
	prevalcode <- 3			
	newcancerdeathcode <- 4		
	oldcancerdeathcode <- 5		
	newnoncancerdeathcode <- 6		
	oldnoncancerdeathcode <- 7		
	if (villagegraph) {
		if (graphfiles && graphtype == "PDF") pdf(file="sacsvillage.pdf", width = 10, height = 10)
		if (graphfiles && graphtype == "PNG") png(filename = "sacsvillage.png", width = 6000, height = 6000, pointsize = 24, bg = "white")
		if (graphfiles && graphtype == "EMF") win.metafile(filename = "sacsvillage.emf", width = 10, height = 10, pointsize = 12)
		if (graphfiles && graphtype == "PS") postscript("sacsvillage.ps", paper="a4", horizontal=FALSE)
		# assign plot basics
		# mar = c(bottom, left, top, right)
		if (graphfiles) par(xpd = TRUE, mar = c(7, 1, 1, 1))
		if (!graphfiles) par(xpd = TRUE, mar = c(8, 1, 2, 1))
		plot(x=seq(from=0, to=100, by=0.5), y=seq(from=0, to=100, by=0.5), type="n", xlab="", ylab="", font=2, axes=F, ann=F, cex.lab=1, bg="white") 	
		#mtext("Typical Distribution of Persons and Disease Status in Small Area Cancer Simulation", side=3, col=1, font=2, cex=1, line=3, outer=F)
		mtext("Street", side=1, col=1, font=2, cex=1, line=2, outer=F)
		box()
		axis(side=1, labels=LETTERS[1:10], at=seq(from=3, to=97.5, by=10.5), font=2, tick=T)
		xboxsize <- 1.2
		yboxsize <- 1.4
		textxdist <- 2.5
		if (graphfiles) {
			legendytop <- -15
			legendybot <- -17
		}
		if (!graphfiles) {
			legendytop <- -10
			legendybot <- -13
		}
		legendytoptxt <- legendytop + 0.5
		legendybottxt <- legendybot + 0.5
		textcex <- 0.7
		init.cancercolvec <- rainbow(24)
		cancercolvec <- c(init.cancercolvec[5], init.cancercolvec[6], init.cancercolvec[13], init.cancercolvec[18])
		fillcolvec <- c("white", cancercolvec, "grey", "black")
		textcolvec <- c("black", "black", "black", "black", "white", "black", "white")

		legendxvec <- c(0, 22, 56, 68)
		ngon.d.circle <- 3.5
		ngon.d.square <- 4.3
		x.correction <- 0.8
		y.correction <- 0.6

		# outer legend frame
		rect((legendxvec[1]-1),(legendybot-2),102,(legendytop+3), border="black", col="white")

		# block 1
		text(x=(legendxvec[1]-0.1), y=(legendytoptxt+0.1), labels="53", side=1, col=1, font=2, cex=0.7, outer=F, adj=0)
		text(x=(legendxvec[1]+textxdist), y=legendytoptxt, labels="current/last person age", side=1, col=1, font=1, cex=textcex, outer=F, adj=0)

		ngon(xydc=c(legendxvec[1]+x.correction, legendybot+y.correction, ngon.d.square, "black"), n=4, type=2)
		text(x=(legendxvec[1]+textxdist), y=legendybottxt, labels="healthy", side=1, col=1, font=1, cex=textcex, outer=F, adj=0)

		# block 2
		ngon(xydc=c(legendxvec[2]+x.correction,legendytop+y.correction, ngon.d.square, fillcolvec[6]), n=4, type=3)
		text(x=(legendxvec[2]+textxdist), y=legendytoptxt, labels="non-cancer death in current year", side=1, col=1, font=1, cex=textcex, outer=F, adj=0)

		ngon(xydc=c(legendxvec[2]+x.correction,legendybot+y.correction, ngon.d.square, fillcolvec[7]), n=4, type=3)
		text(x=(legendxvec[2]+textxdist), y=legendybottxt, labels="non-cancer death within last five years", side=1, col=1, font=1, cex=textcex, outer=F, adj=0)

		# block 3
		ngon(xydc=c(legendxvec[3]+x.correction,legendytop+y.correction, ngon.d.circle, fillcolvec[2]), n=10, type=3)
		text(x=(legendxvec[3]+textxdist), y=legendytoptxt, labels="incident", side=1, col=1, font=1, cex=textcex, outer=F, adj=0)

		ngon(xydc=c(legendxvec[3]+x.correction,legendybot+y.correction, ngon.d.circle, fillcolvec[3]), n=10, type=3)
		text(x=(legendxvec[3]+textxdist), y=legendybottxt, labels="prevalent", side=1, col=1, font=1, cex=textcex, outer=F, adj=0)

		# block 4
		ngon(xydc=c(legendxvec[4]+x.correction,legendytop+y.correction, ngon.d.circle, fillcolvec[4]), n=10, type=3)
		text(x=(legendxvec[4]+textxdist), y=legendytoptxt, labels="cancer-related death in current year", side=1, col=1, font=1, cex=textcex, outer=F, adj=0)

		ngon(xydc=c(legendxvec[4]+x.correction,legendybot+y.correction, ngon.d.circle, fillcolvec[5]), n=10, type=3)
		text(x=(legendxvec[4]+textxdist), y=legendybottxt, labels="cancer-related death within last five years", side=1, col=1, font=1, cex=textcex, outer=F, adj=0)

		colvec <- rainbow(nhouse, start=0, end=0.85)
	}
	# assign start age vector.
	tmppopclasssizevec <- c()
	for (i in 1:101) tmppopclasssizevec <- c(tmppopclasssizevec, sum(rbinom(startn, size=1, prob=popclasssizeweights[i])))
	empn <- sum(tmppopclasssizevec)
	relfreqpopclasssizevec <- tmppopclasssizevec / empn
	popclasssizevec <- round(relfreqpopclasssizevec * startn)
	N <- sum(popclasssizevec)
	agevec <- c(rep(c(0:100), popclasssizevec))
	# assign house distribution of persons
	where <- floor(runif(N, min=1, max=nhouse+0.999999))
	# calc street code of persons
	street <- ceiling(where/10)
	# assign house plotting vars
	housexdist <- 10.4
	houseydist <- 10.4
	# assign vectors for plotting
	popsizevec <-  c()
	birthratevec <- c()
	meanagevec <- c()
	year.agevec <- c()
	incidencevec <- c()
	meanincidenceagevec <- c()
	year.incidenceagevec <- c()
	prevalencevec <- c()
	meanprevalenceagevec <- c()
	year.prevalenceagevec <- c()
	cancermortalityvec <- c()
	noncancermortalityvec <- c()
	meancancerdeathagevec <- c()
	year.cancerdeathagevec <- c()
	meannoncancerdeathagevec <- c()
	year.noncancerdeathagevec <- c()
	cancer5ysurvprobvec <- c()
	noncancernysurvprobvec <- c()
	propaffectedhousesperstreetvec <- c()
	medianpropaffectedhousesperstreetvec <- c()
	propstreetswithallhousesaffectedvec <- c()
	yearvec <- c()
	boxplotyearvec <- c()
	# assign id vector.
	idvec <- c(1:N)
	# assign status vector
	statusvec <- c(rep(healthcode, N))
	# assign vector of time since cancer onset.
	timesincediag <- c(rep(0, N))
	# assign vector of time since death.
	timesincedeath <- c(rep(0, N))
	year <- 1
	nextid <- N + 1
	absolcuminc <- 0
	absolcumcancerrelmort <- 0
	while (year <= nyears && N > 2) {
		# deceased persons leave population after end of 'memory period'.
		idvec <- idvec[timesincedeath <= memtime]
		agevec <- agevec[timesincedeath <= memtime]
		where <- where[timesincedeath <= memtime]
		street <- street[timesincedeath <= memtime]
		timesincediag <- timesincediag[timesincedeath <= memtime]
		statusvec <- statusvec[timesincedeath <= memtime]
		timesincedeath <- timesincedeath[timesincedeath <= memtime]
		# calc size of living population at beginning of year.
		N <- length(statusvec[statusvec <= prevalcode])
		###################################################
		# birth process
		###################################################
		absolutebirthrate <- 0
		if (birthproc) {
			absolutebirthrate <- sum(rbinom(N, size=1, prob=brate))
			if (absolutebirthrate > 0) {
				idvec <- c(idvec, nextid:(nextid+absolutebirthrate-1))
				nextid <- nextid + absolutebirthrate
				agevec <- c(agevec, c(rep(0, absolutebirthrate)))
				where <- c(where, floor(runif(absolutebirthrate, min=1, max=nhouse+0.999999)))
				street <- ceiling(where/10)
				statusvec <- c(statusvec, c(rep(healthcode, absolutebirthrate)))	
				timesincediag <- c(timesincediag, c(rep(0, absolutebirthrate)))
				timesincedeath <- c(timesincedeath, c(rep(0, absolutebirthrate)))
			}
		}
		# calc size of living population after births.
		N <- length(statusvec[statusvec <= prevalcode])
		# pause function (currently not required)
		#t.end <- proc.time()[3] + 3 ## the time this loop should end at 
		#while (proc.time()[3] < t.end) {}
		###################################################
		# initial prevalence process
		###################################################
		if (year == 1 && length(statusvec[statusvec == healthcode]) > 1) {
			absolutestartprevalence <- 0
			absolutestartprev <- 0
			try(absolutestartprevalence <- sum(rbinom(length(statusvec[statusvec == healthcode]), size=1, prob=0.012)))
			if (absolutestartprevalence > 0 && absolutestartprevalence < length(statusvec[statusvec == healthcode])) {
				if (length(idvec[statusvec == healthcode]) > 1) {
					phealthidvec <- idvec[statusvec == healthcode]
					phealthstatusvec <- statusvec[statusvec == healthcode]
					phealthagevec <- agevec[statusvec == healthcode]
					phealthtimesincediag <- timesincediag[statusvec == healthcode]
					phealthweightvec <- c(rep(NA, length(phealthstatusvec)))
					for (i in 1:length(incidweighttab$weights)) {
						phealthweightvec[phealthagevec >= incidweighttab$min[i] & phealthagevec <= incidweighttab$max[i]] <- incidweighttab$weights[i]
					}
					if (length(phealthidvec) != length(phealthweightvec)) stop("unequal length of idvec and weightvec!")
					incidid <- NA
					try(incidid <- sample(phealthidvec, size=absolutestartprevalence, replace=F, prob=phealthweightvec))
					if (!is.na(incidid)) {
						for (i in 1:absolutestartprevalence) {
							phealthtimesincediag[phealthidvec == incidid[i]] <- floor(runif(1, min=0, max=5.999999))
							phealthstatusvec[phealthidvec == incidid[i]] <- prevalcode
						}
					}
					timesincediag[statusvec == healthcode] <- phealthtimesincediag
					statusvec[statusvec == healthcode] <- phealthstatusvec
					# calc absolute start prevalence.
					absolutestartprev <- length(statusvec[statusvec == prevalcode])
					# check to see if all went well.
					if (absolutestartprev != absolutestartprevalence) stop("error in calculation of start prevalence!")
				}
			}
		}
		###################################################
		# incidence process
		###################################################
		if (length(statusvec[statusvec == healthcode]) > 1) {
			absoluteincidence <- 0
			absoluteincid <- 0
			try(absoluteincidence <- sum(rbinom(length(statusvec[statusvec == healthcode]), size=1, prob=incid)))
			if (absoluteincidence > 0 && absoluteincidence < length(statusvec[statusvec == healthcode])) {
				if (length(idvec[statusvec == healthcode]) > 1) {
					ihealthidvec <- idvec[statusvec == healthcode]
					ihealthstatusvec <- statusvec[statusvec == healthcode]
					ihealthagevec <- agevec[statusvec == healthcode]
					ihealthweightvec <- c(rep(NA, length(ihealthstatusvec)))
					for (i in 1:length(incidweighttab$weights)) {
						ihealthweightvec[ihealthagevec >= incidweighttab$min[i] & ihealthagevec <= incidweighttab$max[i]] <- incidweighttab$weights[i]
					}
					if (length(ihealthidvec) != length(ihealthweightvec)) stop("unequal length of idvec and weightvec!")
					incidid <- NA
					try(incidid <- sample(ihealthidvec, size=absoluteincidence, replace=F, prob=ihealthweightvec))
					if (!is.na(incidid)) {
						for (i in 1:absoluteincidence) {
							ihealthstatusvec[ihealthidvec == incidid[i]] <- incidcode
						}
					}
					statusvec[statusvec == healthcode] <- ihealthstatusvec
					# calc absolute incidence.
					absoluteincid <- length(statusvec[statusvec == incidcode])
					# check to see if all went well.
					if (absoluteincid != absoluteincidence) stop("error in calculation of incidence!")
				}
			}
		}
		###################################################
		# cancer-related mortality process
		###################################################
		if (length(statusvec[statusvec <= prevalcode]) > 1) {
			absolutecancermort <- 0
			newcancerabsolutemortality <- 0
			try(absolutecancermort <- sum(rbinom(length(statusvec[statusvec <= prevalcode]), size=1, prob=cancrelmort)))
			if (absolutecancermort > 0 && absolutecancermort < length(statusvec[statusvec == incidcode | statusvec == prevalcode])) {
				if (length(idvec[statusvec == incidcode | statusvec == prevalcode]) > 1) {
					prevalidvec <- idvec[statusvec == incidcode | statusvec == prevalcode]
					prevalstatusvec <- statusvec[statusvec == incidcode | statusvec == prevalcode]
					prevalagevec <- agevec[statusvec == incidcode | statusvec == prevalcode]
					cancrelweightvec <- c(rep(NA, length(prevalstatusvec)))
					for (i in 1:length(cancreldeathweighttab$weights)) {
						cancrelweightvec[prevalagevec >= cancreldeathweighttab$min[i] & prevalagevec <= cancreldeathweighttab$max[i]] <- cancreldeathweighttab$weights[i]
					}
					# check to see if all weights are correctly assigned.
					if (any(is.na(cancrelweightvec))) stop("cancrelweightvec contains >= 1 NA values")
					if (length(prevalidvec) != length(cancrelweightvec)) stop("unequal length of idvec and weightvec!")
					cdeathid <- NA
					try(cdeathid <- sample(prevalidvec, size=absolutecancermort, replace=F, prob=cancrelweightvec))
					if (!is.na(cdeathid)) {
						for (i in 1:absolutecancermort) {
							prevalstatusvec[prevalidvec == cdeathid[i]] <- newcancerdeathcode
						}
					}	
					statusvec[statusvec == incidcode | statusvec == prevalcode] <- prevalstatusvec
					# calc absolute cancer-related mortality.
					newcancerabsolutemortality <- length(statusvec[statusvec == newcancerdeathcode])
					# check to see if all went well.
					if (newcancerabsolutemortality != absolutecancermort) {
						cat("newcancerabsolutemortality:", newcancerabsolutemortality, ", absolutecancermort:", absolutecancermort, "\n")
						stop("error in calculation of cancer deaths!")
					}
				}
			}
		}
		###################################################
		# non-cancer mortality process
		###################################################
		if (length(statusvec[statusvec <= prevalcode]) > 1) {
			absolutenoncancermort <- 0
			newnoncancerabsolutemortality <- 0
			try(absolutenoncancermort <- sum(rbinom(length(statusvec[statusvec <= prevalcode]), size=1, prob=noncancrelmort)))
			if (absolutenoncancermort > 0 && absolutenoncancermort < length(statusvec[statusvec <= prevalcode])) {
				if (length(idvec[statusvec <= prevalcode]) > 1) {
					dhealthidvec <- idvec[statusvec <= prevalcode]
					dhealthstatusvec <- statusvec[statusvec <= prevalcode]
					dhealthagevec <- agevec[statusvec <= prevalcode]
					noncancrelweightvec <- c(rep(NA, length(dhealthstatusvec)))
					for (i in 1:length(noncancreldeathweighttab$weights)) {
						noncancrelweightvec[dhealthagevec >= noncancreldeathweighttab$min[i] & dhealthagevec <= noncancreldeathweighttab$max[i]] <- noncancreldeathweighttab$weights[i]
					}
					# check to see if all weights are correctly assigned.
					if (any(is.na(noncancrelweightvec))) stop("noncancrelweightvec contains >= 1 NA values")
					if (length(dhealthidvec) != length(noncancrelweightvec)) stop("unequal length of idvec and weightvec!")
					ncdeathid <- NA
					try(ncdeathid <- sample(dhealthidvec, size=absolutenoncancermort, replace=F, prob=noncancrelweightvec))
					if (!is.na(ncdeathid)) {
						for (i in 1:absolutenoncancermort) {
							dhealthstatusvec[dhealthidvec == ncdeathid[i]] <- newnoncancerdeathcode
						}	
					}
					statusvec[statusvec <= prevalcode] <- dhealthstatusvec
					# calc absolute all-cause mortality.
					newnoncancerabsolutemortality <- length(statusvec[statusvec == newnoncancerdeathcode])
					# check to see if all went well.
					if (newnoncancerabsolutemortality != absolutenoncancermort) {
						cat("newnoncancerabsolutemortality:", newnoncancerabsolutemortality, ", absolutenoncancermort:", absolutenoncancermort, "\n")
						stop("error in calculation of non-cancer deaths!")
					}
				}
			}
		}
		# calc absolute prevalence.
		absoluteprevalence <- length(statusvec[statusvec == prevalcode])
		if (verbose >= 1) {
			cat("year:", year, ", N:", N, ", absol. birthrate:", absolutebirthrate, ", absol. incidence:", absoluteincidence, ", absol. prevalence:", absoluteprevalence, 
				", absol. cancer mortality:", newcancerabsolutemortality, ", absol. non-cancer mortality:", newnoncancerabsolutemortality, "\n")
		}
		if (year > runintime) {
			# append to yearvec
			yearvec <- c(yearvec, year)
			# append to birthratevec
			popsizevec <- c(popsizevec, N)
			birthrate <- absolutebirthrate * 100000 / N
			birthratevec <- c(birthratevec, birthrate)
			# append to meanagevec
			meanagevec <- c(meanagevec, mean(agevec[statusvec <= prevalcode]))
			year.agevec <- c(year.agevec, agevec[statusvec <= prevalcode])
			# append to propaffectedhousesperstreetvec  (used for plotting)
			psychmemtime <- 5
			nstreet <- 10
			nhouseperstreet <- 10
			hitsperstreetvec <- c()
			for (i in 1:nstreet) {
				bpwhere <- where[timesincedeath <= psychmemtime]
				bpstatus <- statusvec[timesincedeath <= psychmemtime]
				bpstreet <- street[timesincedeath <= psychmemtime]
				streetwhere <- bpwhere[bpstreet == i]
				streetstat <- bpstatus[bpstreet == i]
				hitsperstreet <- 0
				for (j in 1:nhouseperstreet) {
					houseid <- ((i-1)*10) + j
					housestat <- streetstat[streetwhere == houseid]
					currhousehits <- length(housestat[housestat == incidcode | housestat == prevalcode | 
									housestat == newcancerdeathcode | housestat == oldcancerdeathcode])
					if (currhousehits > 0) hitsperstreet <- hitsperstreet + 1
				}
				hitsperstreetvec <- c(hitsperstreetvec, hitsperstreet)
			}
			# for each street, calc prop. of affected houses per year of simulation.
			propaffectedhousesperstreet <- hitsperstreetvec / nhouseperstreet
			# append to vector that contains all proportions for all streets over all sim. years.
			propaffectedhousesperstreetvec <- c(propaffectedhousesperstreetvec, propaffectedhousesperstreet)
			boxplotyearvec <- c(boxplotyearvec, c(rep(year, nstreet)))
			# calc median prop. of affected houses in village per year of simulation.
			# append to medianpropaffectedhousesperstreetvec (used for simulation)
			medianpropaffectedhousesperstreetvec <- c(medianpropaffectedhousesperstreetvec, median(propaffectedhousesperstreet))
			# append to noncancermortalityvec
			incidence <- absoluteincidence * 100000 / N
			incidencevec <- c(incidencevec, incidence)
			# append to prevalencevec
			prevalence <- absoluteprevalence * 100000 / N 
			prevalencevec <- c(prevalencevec, prevalence)
			# append to cancermortalityvec
			cancermortality <- newcancerabsolutemortality * 100000 / N 
			cancermortalityvec <- c(cancermortalityvec, cancermortality)
			# append to noncancermortalityvec
			noncancermortality <- newnoncancerabsolutemortality * 100000 / N 
			noncancermortalityvec <- c(noncancermortalityvec, noncancermortality)
			# append to meanincidenceagevec	
			incidenceage <- agevec[statusvec == incidcode]
			year.incidenceagevec <- c(year.incidenceagevec, incidenceage)
			meanincidenceagevec <- c(meanincidenceagevec, mean(incidenceage))
			# append to cancer5ysurvprobvec
			crtime <- timesincediag[statusvec > healthcode & statusvec < newnoncancerdeathcode]
			crstat <- statusvec[statusvec > healthcode & statusvec < newnoncancerdeathcode]
			crcens <- c(rep(0, length(crstat)))
			crcens[crstat == newcancerdeathcode] <- 1
			crcens[crstat == oldcancerdeathcode] <- 1
			if (length(crtime) > 0) cancer5ysurvprob <- fsurvprob(zeit=crtime, zens=crcens, tpoint=5)
			if (length(crtime) == 0) cancer5ysurvprob <- NA
			cancer5ysurvprobvec <- c(cancer5ysurvprobvec, cancer5ysurvprob)
			# append to meancancerdeathagevec	
			cancerdeathage <- agevec[statusvec == newcancerdeathcode | statusvec == oldcancerdeathcode]
			mean.cancerdeathage <- mean(cancerdeathage)
			year.cancerdeathagevec <- c(year.cancerdeathagevec, cancerdeathage)
			meancancerdeathagevec <- c(meancancerdeathagevec, mean.cancerdeathage)
			# append to meancancerdeathagevec	
			noncancerdeathage <- agevec[statusvec == newnoncancerdeathcode | statusvec == oldnoncancerdeathcode]
			mean.noncancerdeathage <- mean(noncancerdeathage)
			year.noncancerdeathagevec <- c(year.noncancerdeathagevec, noncancerdeathage)
			meannoncancerdeathagevec <- c(meannoncancerdeathagevec, mean.noncancerdeathage)
			#cat("year:", year, ", mean cancer-related life shortening:", (meannoncancerdeathage-meancancerdeathage), "\n")

			if (numericgraph) {
				if (graphfiles && year == nyears) {
					if (graphtype == "PDF") pdf(file="sacsnumeric.pdf", width = 12, height = 8)
					if (graphtype == "PNG") png(filename = "sacsnumeric.png", width = 6000, height = 4000, bg = "white")
					if (graphtype == "EMF") win.metafile(filename = "sacsnumeric.emf", width = 21, height = 14, pointsize = 12)
					if (graphtype == "PS") postscript("sacsnumeric.ps", paper="a4", horizontal=FALSE)
				}
				# mar = c(bottom, left, top, right)
				plotnrow <- 4
				plotncol <- 3
				par(mfrow=c(plotnrow, plotncol), font = 2, oma = c(0, 0, 0, 0), mar=c(2, 4, 2, 2)) 	 
				cextitle <- 1
				cexlab <- 1
				cexaxis <- 1
				linewidth <- 2
				plottype <- "l"
				#############
				# graph row 1
				##############################################################
				# graph 1: N
				##############################################################
				plot(yearvec, popsizevec, type=plottype, main="population size by year", 
					ylab="N", xlab="year", cex.main=cextitle, cex.lab=cexlab, cex.axis=cexaxis, lwd=linewidth, bg="white")
				##############################################################
				# graph 2: birth rate
				##############################################################
				plot(yearvec, birthratevec, type=plottype, main="birthrate by year", 
					ylab="rate per 100,000", xlab="year", cex.main=cextitle, cex.lab=cexlab, cex.axis=cexaxis, lwd=linewidth, bg="white")
				##############################################################
				# graph 3: mean age
				##############################################################
				plot(yearvec, meanagevec, type=plottype, main="mean population age by year", 
					ylab="age (years)", xlab="year", ylim=c(0,100), cex.main=cextitle, cex.lab=cexlab, cex.axis=cexaxis, lwd=linewidth, bg="white")
				#############
				# graph row 2
				##############################################################
				# graph 4: incidence
				##############################################################
				plot(yearvec, incidencevec, type=plottype, main="cancer incidence by year", 
					ylab="incidence per 100,000", xlab="year", cex.main=cextitle, cex.lab=cexlab, cex.axis=cexaxis, lwd=linewidth, bg="white")
				##############################################################
				# graph 5: prevalence
				##############################################################
				plot(yearvec, prevalencevec, type=plottype, main="cancer prevalence by year", 
					ylab="prevalence per 100,000", xlab="year", cex.main=cextitle, cex.lab=cexlab, cex.axis=cexaxis, lwd=linewidth, bg="white")
				##############################################################
				# graph 6: proportion of houses per street with >= 1 cancer patient.
				##############################################################
				boxplot(propaffectedhousesperstreetvec ~ boxplotyearvec, range=0, main="houses per street with >= 1 cancer patient", 
						ylab="proportion", xlab="year", ylim=c(0,1), cex.main=cextitle, cex.lab=cexlab, cex.axis=cexaxis, lwd=linewidth, bg="white")
				#############
				# graph row 3
				##############################################################
				# graph 7: mean incidence age 
				##############################################################
				plot(yearvec, meanincidenceagevec, type=plottype, main="mean incidence age by year", 
					ylab="age (years)", xlab="year", ylim=c(0,100), cex.main=cextitle, cex.lab=cexlab, cex.axis=cexaxis, lwd=linewidth, bg="white")
				##############################################################
				# graph 8: cancer-related mortality
				##############################################################
				plot(yearvec, cancermortalityvec, type=plottype, main="cancer-related mortality by year", 
					ylab="mortality per 100,000", xlab="year", cex.main=cextitle, cex.lab=cexlab, cex.axis=cexaxis, lwd=linewidth, bg="white")
				##############################################################
				# graph 9: non-cancer mortality
				##############################################################
				plot(yearvec, noncancermortalityvec, type=plottype, main="non-cancer mortality by year", 
					ylab="mortality per 100,000", xlab="year", cex.main=cextitle, cex.lab=cexlab, cex.axis=cexaxis, lwd=linewidth, bg="white")
				#############
				# graph row 4
				##############################################################
				# graph 10: 5-years survival probability of diseased patients
				##############################################################
				plot(yearvec, cancer5ysurvprobvec, type=plottype, main="cancer-related 5-years survival by year", 
					ylab="survival probability", xlab="year", ylim=c(0,1), cex.main=cextitle, cex.lab=cexlab, cex.axis=cexaxis, lwd=linewidth, bg="white")
				##############################################################
				# graph 11: mean age at time of death for cancer patients.
				##############################################################
				plot(yearvec, meancancerdeathagevec, type=plottype, main="mean cancer-related dying age by year", 
					ylab="age (years)", xlab="year", ylim=c(0,100), cex.main=cextitle, cex.lab=cexlab, cex.axis=cexaxis, lwd=linewidth, bg="white")
		             ##############################################################
				# graph 12: mean age at time of death for non-diseased persons. 
				##############################################################
				plot(yearvec, meannoncancerdeathagevec, type=plottype, main="mean non-cancer dying age by year", 
					ylab="age (years)", xlab="year", ylim=c(0,100), cex.main=cextitle, cex.lab=cexlab, cex.axis=cexaxis, lwd=linewidth, bg="white")
			}
			##############################################################
			# calculate number of streets with >= 1 cancer patient in every house.
			##############################################################
			nstreet <- 10
			nhouseperstreet <- 10
			nstreetswithallhousesaffected <- 0
			for (i in 1:nstreet) {
				bpwhere <- where[timesincedeath <= psychmemtime]
				bpstatus <- statusvec[timesincedeath <= psychmemtime]
				bpstreet <- street[timesincedeath <= psychmemtime]
				streetwhere <- bpwhere[bpstreet == i]
				streetstat <- bpstatus[bpstreet == i]
				hitsperstreet <- 0
				for (j in 1:nhouseperstreet) {
					houseid <- ((i-1)*10) + j
					housestat <- streetstat[streetwhere == houseid]
					currhousehits <- length(housestat[housestat == incidcode | housestat == prevalcode | 
									housestat == newcancerdeathcode | housestat == oldcancerdeathcode])
					if (currhousehits > 0) hitsperstreet <- hitsperstreet + 1
				}
				if (hitsperstreet == nstreet) nstreetswithallhousesaffected <- nstreetswithallhousesaffected + 1
			}
			propstreetswithallhousesaffectedvec <- c(propstreetswithallhousesaffectedvec, (nstreetswithallhousesaffected / nstreet))
			##############################################################
			# As this variable is almost always = 0, don't plot it!
			##############################################################
			#if (numericgraph) plot(yearvec, propstreetswithallhousesaffectedvec, type=plottype, main="streets with >= 1 cancer patient in every house", 
			#						ylab="proportion", xlab="year", , cex.main=cextitle, cex.lab=cexlab, cex.axis=cexaxis, lwd=linewidth, bg="white")
			##############################################################
			# calculate mean prevalence age.
			##############################################################
			meanprevalenceage <- mean(agevec[statusvec == prevalcode])
			year.prevalenceagevec <- c(year.prevalenceagevec, agevec[statusvec == prevalcode])
			meanprevalenceagevec <- c(meanprevalenceagevec, meanprevalenceage)
			if (villagegraph && (!graphfiles || (graphfiles && year == nyears))) {
				housex <- -1
				housey <- -2
				housecount <- 0
				vstart <- 1
				# mar = c(bottom, left, top, right)
				# in years > 1, delete previous subtitle by overwriting with white color
				if (!graphfiles && year > 1) mtext(paste("Year #", year-1, sep=""), side=3, col="white", font=2, cex=0.8, line=1, outer=F)
				# print subtitle
				if (!graphfiles) mtext(paste("Year #", year, sep=""), side=3, col=1, font=2, cex=0.8, line=1, outer=F)
				for (i in 1:nhouse) {
					status <- statusvec[where == i]
					age <- agevec[where == i]
					if (year > 1) prevage <- prevagevec[prevwhere == i]
					fhouse(housex, housey, farbe=colvec[i], year, age=age, prevage=prevage, status=status, 
						nyears=nyears, fillcolvec=fillcolvec, textcolvec=textcolvec)
					housey <- housey + houseydist
					housecount <- housecount + 1
					if (housecount == 10) {
						housex <- housex + housexdist
						housey <- -2
						housecount <- 0
					}
				}
			}
			#if (graphfiles && numericgraph && year == nyears) mtext("Numerical Characteristics of Typical Course of Small Area Cancer Simulation", side=3, col=1, font=2, cex=1, line=1, outer=T)
			##############################################################
			# switch off graphics device, if required
			##############################################################
			if (graphfiles && year == nyears) dev.off()
		}
		prevagevec <- agevec
		prevwhere <- where
		# calc cum abs inc over total sim period
		absolcuminc <- absolcuminc + absoluteincidence
		# calc cum abs canc-rel mort over total sim period
		absolcumcancerrelmort <- absolcumcancerrelmort + newcancerabsolutemortality
		# change incidcode to prevalcode at end of year.
		statusvec[statusvec == incidcode] <- prevalcode
		# change newdeathcode to olddeathcode at end of year.
		statusvec[statusvec == newcancerdeathcode] <- oldcancerdeathcode
		statusvec[statusvec == newnoncancerdeathcode] <- oldnoncancerdeathcode
		# increase age variable in living persons by one at end of year
		agevec[statusvec <= prevalcode] <- agevec[statusvec <= prevalcode] + 1
		# increase survival time variable in living persons by one at end of year
		timesincediag[statusvec == prevalcode] <- timesincediag[statusvec == prevalcode] + 1
		# increase time since death variable in dead persons by one at end of year
		timesincedeath[statusvec == oldcancerdeathcode] <- timesincedeath[statusvec == oldcancerdeathcode] + 1
		timesincedeath[statusvec == oldnoncancerdeathcode] <- timesincedeath[statusvec == oldnoncancerdeathcode] + 1
		# increase year by one.
		year <- year + 1
	}
	#cat("pooled mean cancer-related life shortening:", (mean(meannoncancerdeathagevec, na.rm = TRUE) - mean(meancancerdeathagevec, na.rm = TRUE)), "\n")
	outlist <- list(year.popsize=popsizevec,
				meanpopsize=mean(popsizevec), 
				year.birthrate=birthratevec,
				meanbirthrate=mean(birthratevec), 
				year.age=meanagevec,
				meanage=mean(meanagevec), 
				year.incidence=incidencevec,
				meanincidence=mean(incidencevec), 	
				year.incidenceage=meanincidenceagevec,
				meanincidenceage=mean(meanincidenceagevec, na.rm = TRUE), 
				year.prevalence=prevalencevec,
				meanprevalence=mean(prevalencevec), 
				year.prevalenceage=meanprevalenceagevec,
				meanprevalenceage=mean(meanprevalenceagevec, na.rm = TRUE), 
				#the following vector takes all prop. of affected houses per street over sim. period (e.g., 10 streets x 50 yrs. = 500 props.).
				total.propaffectedhousesperstreet=propaffectedhousesperstreetvec, 
				#the following scalar takes the median over 50 years of the median prop. of affected houses per street per year.
				medianpropaffectedhousesperstreet=median(medianpropaffectedhousesperstreetvec), 
				year.propstreetswithallhousesaffected=propstreetswithallhousesaffectedvec, 
				meanpropstreetswithallhousesaffected=mean(propstreetswithallhousesaffectedvec), 
				year.cancerdeathage=meancancerdeathagevec,
				meancancerdeathage=mean(meancancerdeathagevec, na.rm = TRUE), 
				year.noncancerdeathage=meannoncancerdeathagevec,
				meannoncancerdeathage=mean(meannoncancerdeathagevec, na.rm = TRUE), 
				year.cancermortality=cancermortalityvec,
				meancancermortality=mean(cancermortalityvec), 
				year.noncancermortality=noncancermortalityvec,
				meannoncancermortality=mean(noncancermortalityvec), 
				year.cancer5yrssurvprob=cancer5ysurvprobvec,
				meancancer5yrssurvprob=mean(cancer5ysurvprobvec, na.rm = TRUE))
}

nsim <- 10000

sim <- F
if (sim) {

# Obtain unique random number seed.
if (file.exists("seed")) {
	seed.tab <- read.table("seed", header = FALSE, sep = "", as.is = FALSE)	
	seed.nrow <- nrow(seed.tab)
	seed <- as.numeric(seed.tab[seed.nrow, 1])
	seed <- seed + 1
	write(seed, file = "seed", ncolumns = 1, append = TRUE)	
}
if (!file.exists("seed")) {
	seed <- 100
	write(seed, file = "seed", ncolumns = 1, append = FALSE)	
}
cat("seed:", seed, "\n")
set.seed(seed)

# assign vectors for sim results
popsize <- c()
meanpopsize <- c()
birthrate <- c()
meanbirthrate <- c()
age <- c()
meanage <- c()
incidence <- c()
meanincidence <- c()
incidenceage <- c()
meanincidenceage <- c()
prevalence <- c()
meanprevalence <- c()
prevalenceage <- c()
meanprevalenceage <- c()
propaffectedhousesperstreet <- c()
medianpropaffectedhousesperstreet <- c()
propstreetswithallhousesaffected <- c()
meanpropstreetswithallhousesaffected <- c()
cancerdeathage <- c()
meancancerdeathage <- c()
noncancerdeathage <- c()
meannoncancerdeathage <- c()
cancermortality <- c()
meancancermortality <- c()
noncancermortality <- c()
meannoncancermortality <- c()
cancer5ysurvprob <- c()
meancancer5ysurvprob <- c()

for (i in 1:nsim) {
	cat("simulation #", i, "of", nsim, "running, please wait...", "\n")
	res <- sacs(startn=500, nhouse=100, nyears=50, incid=0.0047, cancrelmort=0.00288, noncancrelmort=0.00878,
				birthproc=T, brate=0.01, memtime=5, villagegraph=F, numericgraph=F, runintime=0, graphfiles=F, graphtype="PS")
	
	popsize <- c(popsize, res$year.popsize)
	meanpopsize <- c(meanpopsize, res$meanpopsize)

	birthrate <- c(birthrate, res$year.birthrate)
	meanbirthrate <- c(meanbirthrate, res$meanbirthrate)

	age <- c(age, res$year.age)
	meanage <- c(meanage, res$meanage)

	incidence <- c(incidence, res$year.incidence)
	meanincidence <- c(meanincidence, res$meanincidence)

	incidenceage <- c(incidenceage, res$year.incidenceage)
	meanincidenceage <- c(meanincidenceage, res$meanincidenceage)

	prevalence <- c(prevalence, res$year.prevalence)
	meanprevalence <- c(meanprevalence, res$meanprevalence)

	prevalenceage <- c(prevalenceage, res$year.prevalenceage)
	meanprevalenceage <- c(meanprevalenceage, res$meanprevalenceage)

	propaffectedhousesperstreet <- c(propaffectedhousesperstreet, res$total.propaffectedhousesperstreet)
	medianpropaffectedhousesperstreet <- c(medianpropaffectedhousesperstreet, res$medianpropaffectedhousesperstreet)

	propstreetswithallhousesaffected <- c(propstreetswithallhousesaffected, res$year.propstreetswithallhousesaffected)
	meanpropstreetswithallhousesaffected <- c(meanpropstreetswithallhousesaffected, res$meanpropstreetswithallhousesaffected)

	cancerdeathage <- c(cancerdeathage, res$year.cancerdeathage)
	meancancerdeathage <- c(meancancerdeathage, res$meancancerdeathage)

	noncancerdeathage <- c(noncancerdeathage, res$year.noncancerdeathage)
	meannoncancerdeathage <- c(meannoncancerdeathage, res$meannoncancerdeathage)

	cancermortality <- c(cancermortality, res$year.cancermortality)
	meancancermortality <- c(meancancermortality, res$meancancermortality)

	noncancermortality <- c(noncancermortality, res$year.noncancermortality)
	meannoncancermortality <- c(meannoncancermortality, res$meannoncancermortality)

	cancer5ysurvprob <- c(cancer5ysurvprob, res$year.cancer5yrssurvprob)
	meancancer5ysurvprob <- c(meancancer5ysurvprob, res$meancancer5yrssurvprob)
}

simresult <- list(age=age, meanage=meanage, popsize=popsize, meanpopsize=meanpopsize,
			birthrate=birthrate, meanbirthrate=meanbirthrate, 
			incidence=incidence, meanincidence=meanincidence,
			incidenceage=incidenceage, meanincidenceage=meanincidenceage, 
			prevalence=prevalence, meanprevalence=meanprevalence, 
			prevalenceage=prevalenceage, meanprevalenceage=meanprevalenceage,
			propaffectedhousesperstreet=propaffectedhousesperstreet,
			medianpropaffectedhousesperstreet=medianpropaffectedhousesperstreet, 
			propstreetswithallhousesaffected=propstreetswithallhousesaffected, 
			meanpropstreetswithallhousesaffected=meanpropstreetswithallhousesaffected, 
			noncancerdeathage=noncancerdeathage, meannoncancerdeathage=meannoncancerdeathage, 
			cancerdeathage=cancerdeathage, meancancerdeathage=meancancerdeathage, 
			cancermortality=cancermortality, meancancermortality=meancancermortality, 
			noncancermortality=noncancermortality, meannoncancermortality=meannoncancermortality, 
			cancer5yrssurvprob=cancer5ysurvprob, meancancer5yrssurvprob=meancancer5ysurvprob)

outfilename <- paste("sacs.result.version.4.", seed, ".rda", sep="")
save(simresult, file=outfilename)
}

boxplots <- F
if (boxplots) {
linewidth <- 3
res <- load("sacs.result.version.4.126.rda")
#print(simresult)
graphfiles <- T
graphtype <- "PS"
cextitle <- 1.7
cexlab <- 1.5    # 1
cexaxis <- 1.3     # 0.75
#nsim <- length(simresult$meanage)
#groupvec <- c(rep(1:5, rep(nsim, 5)))

if (graphfiles) {
	if (graphtype == "PDF") pdf(file="sacsbp.pdf", width = 12, height = 8)
	if (graphtype == "PNG") png(filename = "sacsbp.png", width = 6000, height = 4000, bg = "white")
	if (graphtype == "EMF") win.metafile(filename = "sacsbp.emf", width = 21, height = 14, pointsize = 12)
	if (graphtype == "PS") postscript("sacsbp.ps", paper="a4", horizontal=FALSE)
}

# mar = c(bottom, left, top, right)
plotnrow <- 3
plotncol <- 1
par(mfrow=c(plotnrow, plotncol), font = 2, oma = c(0, 0, 0, 0), mar=c(6, 5, 0, 2)) 	 

# plot age boxplot
boxplot(simresult$meanage, simresult$meanincidenceage, simresult$meanprevalenceage, simresult$meancancerdeathage, simresult$meannoncancerdeathage,
		xlim=c(0,100), names=c("population", "at incidence", "during prevalence", "at cancer death", "at non-cancer death"), 
		font=2, bg="white", notch=T, horizontal=F, range=1.5, ylab="mean age (years)",
             cex.sub=cextitle, cex.lab=cexlab, cex.axis=cexaxis)
title(sub="a", cex.sub=cextitle, line=4, font=2)

# plot rate boxplot
boxplot(simresult$birthrate, simresult$incidence, simresult$prevalence, simresult$cancermortality, simresult$noncancermortality,
		xlim=c(0,5000), names=c("birthrate", "incidence", "prevalence", "cancer mortality", "non-cancer mortality"), 		
		font=2, bg="white", notch=T, horizontal=F, range=1.5, ylab="rate per 100,000",
             cex.lab=cexlab, cex.axis=cexaxis)
title(sub="b", cex.sub=cextitle, line=4, font=2)

# plot histogram with proportion of houses per street with >= 1 cancer patient within last 5 years 
hist(simresult$propaffectedhousesperstreet, font=2, bg="white", notch=T, horizontal=F, ylab="n", cex.sub=cextitle, cex.lab=cexlab, 
			cex.axis=cexaxis, xlab="proportion of houses per street with >= 1 cancer patient within last 5 years", main="", 
			labels=T, breaks=seq(from=-0.05, to=1.05, by=0.1), ylim=c(0,1500000))
title(sub="c", cex.sub=cextitle, line=5, font=2)

if (graphfiles) dev.off()

cat("mean population size:", mean(simresult$popsize), ", SE:", sd(simresult$popsize)/sqrt(nsim), "\n") 
cat("mean proportion of affected houses per street:", mean(simresult$medianpropaffectedhousesperstreet), ", SE:", sd(simresult$medianpropaffectedhousesperstreet)/sqrt(nsim), "\n")
cat("mean proportion of streets with all houses affected:", mean(simresult$propstreetswithallhousesaffected), ", SE:", sd(simresult$propstreetswithallhousesaffected)/sqrt(nsim), "\n")
cat("mean cancer-related 5-yr survival probability:", mean(simresult$cancer5yrssurvprob, na.rm=T), ", SE:", sd(simresult$cancer5yrssurvprob, na.rm=T)/sqrt(nsim), "\n") 

}

saat <- 115
numsinglerun <- T
if (numsinglerun) {
set.seed(saat)
res <- sacs(startn=500, nhouse=100, nyears=50, incid=0.0047, cancrelmort=0.00288, noncancrelmort=0.00878,
		birthproc=T, brate=0.01, memtime=5, villagegraph=F, numericgraph=T, runintime=0, graphfiles=T, graphtype="PS")
#print(res)
}

vilsinglerun <- F
if (vilsinglerun) {
set.seed(saat)
res <- sacs(startn=500, nhouse=100, nyears=50, incid=0.0047, cancrelmort=0.00288, noncancrelmort=0.00878,
		birthproc=T, brate=0.01, memtime=5, villagegraph=T, numericgraph=F, runintime=0, graphfiles=T, graphtype="PS")
#print(res)
}
